@@ -1,14 +1,14 @@
-/*	This is pstree written by Fred Hucht (c) 1993-2013	*
+/*	This is pstree written by Fred Hucht (c) 1993-2015	*
  *	EMail: fred AT thp.uni-due.de				*
  *	Feel free to copy and redistribute in terms of the	*
  * 	GNU public license. 					*
  *
- * $Id: pstree.c,v 2.36 2013-04-12 11:47:03+02 fred Exp $
+ * $Id: pstree.c,v 2.39 2015/05/13 12:24:47 fred Exp $
  */
 static char *WhatString[]= {
-  "@(#)pstree $Revision: 2.36 $ by Fred Hucht (C) 1993-2013",
+  "@(#)pstree $Revision: 2.39 $ by Fred Hucht (C) 1993-2015",
   "@(#)EMail: fred AT thp.uni-due.de",
-  "$Id: pstree.c,v 2.36 2013-04-12 11:47:03+02 fred Exp $"
+  "$Id: pstree.c,v 2.39 2015/05/13 12:24:47 fred Exp $"
 };
 
 #define MAXLINE 8192
@@ -29,7 +29,10 @@ extern char *termdef(int, char);
 #  ifdef USE_GETPROCS
 #    define IFNEW(a,b) a
 #    define ProcInfo procsinfo
+#    ifndef _AIX61
+/* workaround contributed by Michael Staats <michael.staats AT gmx.de> */
 extern getprocs(struct procsinfo *, int, struct fdsinfo *, int, pid_t *, int);
+#    endif /*_AIX61*/
 #  else /*USE_GETPROCS*/
 #    define IFNEW(a,b) b
 #    define ProcInfo procinfo
@@ -37,7 +40,10 @@ extern getproc(struct procinfo *, int, int);
 extern getuser(struct procinfo *, int, void *, int);
 #  endif /*USE_GETPROCS*/
 
+#  ifndef _AIX61
+/* workaround contributed by Michael Staats <michael.staats AT gmx.de> */
 extern getargs(struct ProcInfo *, int, char *, int);
+#  endif /*_AIX61*/
 
 /*#  define PSCMD 	"ps -ekf"
   #  define PSFORMAT 	"%s %ld %ld %*20c %*s %[^\n]"*/
@@ -198,17 +204,18 @@ static struct TreeChars TreeChars[] = {
     /**/                                                            "",     "",     ""             }  /*UTF8*/
 }, *C;
 
-int MyPid, NProc, Columns, RootPid;
-short showall = TRUE, soption = FALSE, Uoption = FALSE;
-char *name = "", *str = NULL, *Progname;
-long ipid = -1;
-char *input = NULL;
+static int MyPid, NProc, Columns, RootPid;
+static short showall = TRUE, soption = FALSE, Uoption = FALSE;
+static char *name = "", *str = NULL, *Progname;
+static long ipid = -1;
+static char *input = NULL;
 
-int atLdepth=0;    /* LOPTION - track how deep in the print chain we are */
-int maxLdepth=100; /* LOPTION - will be changed by -l n option */
+static int atLdepth=0;    /* LOPTION - track how deep in the print chain we are */
+static int maxLdepth=100; /* LOPTION - will be changed by -l n option */
+static int compress = FALSE;
 
 #ifdef DEBUG
-int debug = FALSE;
+static int debug = FALSE;
 #endif
 
 struct Proc {
@@ -220,7 +227,7 @@ struct Proc {
 } *P;
 
 #ifdef UID2USER
-void uid2user(uid_t uid, char *name, int len) {
+static void uid2user(uid_t uid, char *name, int len) {
 #define NUMUN 128
   static struct un_ {
     uid_t uid;
@@ -262,7 +269,7 @@ void uid2user(uid_t uid, char *name, int len) {
 #endif
 
 #if defined(_AIX) || defined(___AIX)	/* AIX 3.x / 4.x */
-int GetProcessesDirect(void) {
+static int GetProcessesDirect(void) {
   int i, nproc, maxnproc = 1024;
   
   struct ProcInfo *proc;
@@ -376,7 +383,7 @@ int GetProcessesDirect(void) {
 #endif /* _AIX */
 
 #ifdef __linux
-int GetProcessesDirect(void) {
+static int GetProcessesDirect(void) {
   glob_t globbuf;
   unsigned int i, j;
   
@@ -440,7 +447,7 @@ int GetProcessesDirect(void) {
 }
 #endif /* __linux */
 
-int GetProcesses(void) {
+static int GetProcesses(void) {
   FILE *tn;
   int i = 0;
   char line[MAXLINE], command[] = PSCMD;
@@ -546,7 +553,7 @@ int GetProcesses(void) {
   return i;
 }
 
-int GetRootPid(void) {
+static int GetRootPid(void) {
   int me;
   for (me = 0; me < NProc; me++) {
     if (P[me].pid == 1) return P[me].pid;
@@ -588,8 +595,8 @@ void FixZombies(void) {
       P[me].pid = -1;
 #ifdef DEBUG
       if (debug) fprintf(stderr,
-			 "fixed zombie %s with ppid %d\n",
-			 P[me].cmd, P[me].ppid);
+			 "fixed zombie %s with ppid %ld\n",
+			 P[me].cmd, (long)P[me].ppid);
 #endif
     }
   }
@@ -604,7 +611,7 @@ int get_pid_index(long pid) {
 
 #define EXIST(idx) ((idx) != -1)
 
-void MakeTree(void) {
+static void MakeTree(void) {
   /* Build the process hierarchy. Every process marks itself as first child
    * of it's parent or as sister of first child of it's parent */
   int me;  
@@ -624,14 +631,14 @@ void MakeTree(void) {
   }
 }
 
-void MarkChildren(int me) {
+static void MarkChildren(int me) {
   int child;
   P[me].print = TRUE;
   for (child = P[me].child; EXIST(child); child = P[child].sister)
     MarkChildren(child);
 }
 
-void MarkProcs(void) {
+static void MarkProcs(void) {
   int me;
   for (me = 0; me < NProc; me++) {
     if (showall) {
@@ -675,7 +682,7 @@ void MarkProcs(void) {
   }
 }
 
-void DropProcs(void) {
+static void DropProcs(void) {
   int me;
   for (me = 0; me < NProc; me++) if (P[me].print) {
     int child, sister;
@@ -690,11 +697,12 @@ void DropProcs(void) {
   }
 }
 
-void PrintTree(int idx, const char *head) {
+static void PrintTree(int idx, const char *head) {
   char nhead[MAXLINE], out[4 * MAXLINE], thread[16] = {'\0'};
   int child;
   
   if (head[0] == '\0' && !P[idx].print) return;
+  /*if (!P[idx].print) return;*/
   
   if (P[idx].thcount > 1) snprintf(thread, sizeof(thread), "[%ld]", P[idx].thcount);
  
@@ -721,15 +729,33 @@ void PrintTree(int idx, const char *head) {
   /* Process children */
   snprintf(nhead, sizeof(nhead), "%s%s ", head,
 	   head[0] == '\0' ? "" : EXIST(P[idx].sister) ? C->bar : " ");
-  
-  for (child = P[idx].child; EXIST(child); child = P[child].sister)
+
+  /*
+    if ( compress ) {
+    int c1, c2, flag = 0;
+    for ( c1 = P[idx].child; EXIST(c1); c1 = P[c1].sister ) {
+      for ( c2 = P[c1].sister; EXIST(c2); c2 = P[c2].sister ) {
+	if ( 0 == strcmp(P[c1].cmd, P[c2].cmd) ) {
+	  flag = 1;
+	  printf("%d:%d ", c1, c2);
+	  P[c1].pid = -1;
+	  P[c2].print = FALSE;
+	}
+      }
+    }
+    if ( flag ) printf("\n");
+  }
+  */
+
+  for (child = P[idx].child; EXIST(child); child = P[child].sister) {
     PrintTree(child, nhead);
+  }
 
   --atLdepth;                          /* LOPTION */
 
 }
 
-void Usage(void) {
+static void Usage(void) {
   fprintf(stderr,
 	  "%s\n"
 	  "%s\n\n"
@@ -737,7 +763,7 @@ void Usage(void) {
 #ifdef DEBUG
 	  "[-d] "
 #endif
-	  "[-f file] [-g n] [-u user] [-U] [-s string] [-p pid] [-w] [pid ...]\n"
+	  "[-f file] [-g n] [-l n] [-u user] [-U] [-s string] [-p pid] [-w] [pid ...]\n"
 	  /*"   -a        align output\n"*/
 #ifdef DEBUG
 	  "   -d        print debugging info to stderr\n"
@@ -751,8 +777,7 @@ void Usage(void) {
           "   -s string show only branches containing process with <string> in commandline\n"
           "   -p pid    show only branches containing process <pid>\n"
 	  "   -w        wide output, not truncated to window width\n"
-	  "   pid ...   process ids to start from, default is 1 (init)\n"
-	  "             use 0 to also show kernel processes\n"
+	  "   pid ...   process ids to start from, default is 1 (probably init)\n"
 	  , WhatString[0] + 4, WhatString[1] + 4, Progname, PSCMD);
 #ifdef HAS_PGID
   fprintf(stderr, "\n%sProcess group leaders are marked with '%s%s%s'.\n",
@@ -773,11 +798,14 @@ int main(int argc, char **argv) {
   Progname = strrchr(argv[0],'/');
   Progname = (NULL == Progname) ? argv[0] : Progname + 1;
   
-  while ((ch = getopt(argc, argv, "df:g:hl:p:s:u:Uw?")) != EOF)
+  while ((ch = getopt(argc, argv, "cdf:g:hl:p:s:u:Uw?")) != EOF)
     switch(ch) {
       /*case 'a':
 	align   = TRUE;
 	break;*/
+    case 'c':
+      compress = TRUE;
+      break;
 #ifdef DEBUG
     case 'd':
       debug   = TRUE;
@@ -872,12 +900,12 @@ int main(int argc, char **argv) {
     Columns = atoi((char*)termdef(fileno(stdout),'c'));
 #elif defined(TIOCGWINSZ)
     struct winsize winsize;
-    ioctl(fileno(stdout), TIOCGWINSZ, &winsize);
-    Columns = winsize.ws_col;
+    if ( ioctl(fileno(stdout), TIOCGWINSZ, &winsize) != -1 )
+      Columns = winsize.ws_col;
 #elif defined(TIOCGSIZE)
     struct ttysize ttysize;
-    ioctl(fileno(stdout), TIOCGSIZE, &ttysize);
-    Columns = ttysize.ts_cols;
+    if ( ioctl(fileno(stdout), TIOCGSIZE, &ttysize) != -1 )
+      Columns = ttysize.ts_cols;
 #else
     char *env = getenv("COLUMNS");
     Columns = env ? atoi(env) : 80;
@@ -975,6 +1003,16 @@ int snprintf (char *s, int namesiz, char *format, ...) {
 
 /*
  * $Log: pstree.c,v $
+ * Revision 2.39  2015/05/13 12:24:47  fred
+ * Summary: Don't use uninitialized structs when ioctl() fails, e.g. if run with stdout
+ * redirected. Problem reported by Jan Stary.
+ *
+ * Revision 2.38  2015/04/20 14:50:42  fred
+ * Summary: Added patch for AIX61 contributed by Michael Staats
+ *
+ * Revision 2.37  2015/04/20 10:15:29  fred
+ * Summary: V2.36
+ *
  * Revision 2.36  2013-04-12 11:47:03+02  fred
  * Some processes like apache under a recent Linux were listed with UID
  * root instead of the correct UID, as they use setuid(). We now read the
